

/*
###################################################
###### HR3D: A * B with sparse lightfield 
 */

//Mex
#include "mex.h" 
#include <string>

//####################################
//#### Main mex entry point 
//####################################

void mexFunction(int nlhs, mxArray *plhs[], /* Output variables */
		 int nrhs, const mxArray *prhs[]) /* Input variables */
{
    //AB = AmultB(A,B,int32(sampleidx)); %Single channel, result is rowvec
    
    //###################################################
    //####### Get inputs
    
    //#### All in *row-vec format* ####
    if( nlhs != 1 || nrhs != 3 )
	    mexErrMsgTxt("Wrong input/output parameters.");
    
    //A
    double * A = (double*)mxGetData(prhs[0]);
    const int  *A_dims = mxGetDimensions(prhs[0]);
    int A_sz_layer = A_dims[0];
    int A_sz_t = A_dims[1];
    
    //B
    double * B = (double*)mxGetData(prhs[1]);
    const int  *B_dims = mxGetDimensions(prhs[1]);
    int B_sz_t = B_dims[0];
    int B_sz_layer = B_dims[1];
    
    //Layer int
    int * layerInt = (int*)mxGetData(prhs[2]);
    const int *layerInt_dims = mxGetDimensions(prhs[2]);
    int num_samples = layerInt_dims[1];
    
    //Debug
    //printf("num_samples %d, layerdim %d Bszt %d Bszlayer %d Aszt %d Aszlayer %d \n", num_samples, layerInt_dims[0],
    //                                                                                        B_sz_t, B_sz_layer, 
    //                                                                                        A_sz_t, A_sz_layer);
    
    //Sanity
    if( layerInt_dims[0] != 6 ||
        A_sz_layer != B_sz_layer ||
        A_sz_t != B_sz_t )
	    mexErrMsgTxt("Inputs have wrong size.\n");
    
    
    
    //###################################################
    //####### Allocate result
    
    int ndim = 2;
	int dims[2];
	dims[0] = 1;
	dims[1] = num_samples;
	
	plhs[0] = mxCreateNumericArray(ndim, dims, mxDOUBLE_CLASS, mxREAL);
	if (plhs[0] == NULL)
	    mexErrMsgTxt("Could not create output mxArray.\n");
	
	//output
	double *res = (double *)mxGetPr(plhs[0]);
    
    //###################################################
    //####### Compute result
    
    int backLayerIdx, frontLayerIdx;
    int AIdx, BIdx;
    double res_curr;
    for( int s = 0; s < num_samples; s++ )
    {
        //Get indices
        frontLayerIdx = layerInt[ s * 6 + 2] - 1; //C-idx
        backLayerIdx = layerInt[ s * 6 + 5] - 1; //C-idx
        
        //Multiply and add
        res_curr = 0.0;
        for( int t = 0; t < B_sz_t; t++ )
        {
            //Compute indices
            AIdx = t * A_sz_layer + frontLayerIdx; //(frontLayer, t )
            BIdx = backLayerIdx * B_sz_t + t; //(backLayer, t)
                                
            //Compute
            res_curr += A[AIdx] * B[BIdx];
        } 
        
        //Write back
        res[s] = res_curr;
    }
    
    return;
}



