/*
###################################################
###### HR3D: A * B with sparse lightfield 
*/

//Includes
#include <iostream>
#include <string>
#include <cassert>
#include "matrix_factorization.h"
#include "mex.h"
#include "matrix.h"

//####################################
//#### Main mex entry point 
//####################################

void mexFunction(int nlhs, mxArray *plhs[], /* Output variables */
		 int nrhs, const mxArray *prhs[]) /* Input variables */
{
    //AB = AmultB( single(A), single(B),int32(layerInt)); %Single channel, result is rowvec
    
    //###################################################
    //####### Get inputs
    
    //#### All in *row-vec format* ####
    if( nlhs != 1 || nrhs != 3 )
	    mexErrMsgTxt("Wrong input/output parameters.");
    
    //A
    float * A = (float*)mxGetData(prhs[0]);
    const int  *A_dims = mxGetDimensions(prhs[0]);
    int A_sz_layer = A_dims[0];
    int A_sz_t = A_dims[1];
    
    //B
    float * B = (float*)mxGetData(prhs[1]);
    const int  *B_dims = mxGetDimensions(prhs[1]);
    int B_sz_t = B_dims[0];
    int B_sz_layer = B_dims[1];
    
    //Layer int
    int * layerInt = (int*)mxGetData(prhs[2]);
    const int *layerInt_dims = mxGetDimensions(prhs[2]);
    int num_samples = layerInt_dims[1];
    
    //Debug
    //printf("num_samples %d, layerdim %d Bszt %d Bszlayer %d Aszt %d Aszlayer %d \n", 
    //  	num_samples, layerInt_dims[0], B_sz_t, B_sz_layer, A_sz_t, A_sz_layer);
    
    //Sanity
    if( layerInt_dims[0] != 6 ||
        A_sz_layer != B_sz_layer ||
        A_sz_t != B_sz_t )
	    mexErrMsgTxt("Inputs have wrong size.\n");
    
    //###################################################
    //####### Allocate result
    
    int ndim = 2;
    int dims[2];
    dims[0] = 1;
    dims[1] = num_samples;

    plhs[0] = mxCreateNumericArray(ndim, dims, mxSINGLE_CLASS, mxREAL);
    if (plhs[0] == NULL)
	mexErrMsgTxt("Could not create output mxArray.\n");

    //output
    float *res = (float *)mxGetPr(plhs[0]);
    
    //###################################################
    //####### Compute result
    
    AmultB_cuda(A, B, layerInt, res,  
		num_samples,
		A_sz_layer, B_sz_t);

    return;
}
