/*
###################################################
###### HR3D: A_t * Sparse lightfield times 
*/

//Includes
#include <iostream>
#include <string>
#include <cassert>
#include "matrix_factorization.h"
#include "mex.h"
#include "matrix.h"

//####################################
//#### Main mex entry point 
//####################################

void mexFunction(int nlhs, mxArray *plhs[], /* Output variables */
		 int nrhs, const mxArray *prhs[]) /* Input variables */
{
    //AtL = AtmultL(single(samples), int32(ch), single(A), int32(layerInt))
    
	
    //###################################################
    //####### Get inputs
    
    //#### All in *row-vec format* ####
    if( nlhs != 1 || nrhs != 4 )
	mexErrMsgTxt("Wrong input/output parameters.");
    
    //Get pointers and data
    //Samples
    float * samples = (float*)mxGetData(prhs[0]);
    const int *samples_dims = mxGetDimensions(prhs[0]);
    int num_samples = samples_dims[1];
    
    //Channel
    int ch = *((int*)mxGetData(prhs[1]));
    if( ch < 1 || ch > samples_dims[0] )
	mexErrMsgTxt("Channel out of range.\n");
    
    //A
    float * A = (float*)mxGetData(prhs[2]);
    const int  *A_dims = mxGetDimensions(prhs[2]);
    int A_sz_layer = A_dims[0];
    int A_sz_t = A_dims[1];
    
    //Layer int
    int * layerInt = (int*)mxGetData(prhs[3]);
    const int *layerInt_dims = mxGetDimensions(prhs[3]);
    
    //Sanity
    if( layerInt_dims[0] != 6 ||
        layerInt_dims[1] != num_samples )
	    mexErrMsgTxt("Inputs have wrong size.\n");
    
    //Debug
    //printf("num_samples %d, layerdim %d, Ch %d, Aszt %d Aszlayer %d \n", 
    //			num_samples, layerInt_dims[0], ch, A_sz_t, A_sz_layer);

    //###################################################
    //####### Allocate result
    
    int ndim = 2;
    int dims[2];
    dims[0] = A_sz_t;
    dims[1] = A_sz_layer;
    
    plhs[0] = mxCreateNumericArray(ndim, dims, mxSINGLE_CLASS, mxREAL);
    if (plhs[0] == NULL)
	mexErrMsgTxt("Could not create output mxArray.\n");
    
    //output
    float *res = (float *)mxGetPr(plhs[0]);
    
    //###################################################
    //####### Compute result
	
    AtmultL_cuda( A, samples, layerInt, res, 
		  ch,
		  num_samples, samples_dims[0],
		  A_sz_layer, A_sz_t);

    //C++ debug
    /*
    //-------------------------------------------------------------------------
    std::cerr << "Reading input...\n";
    mxArray const *src = prhs[0];
    float *p_src = (float*)mxGetData( src );
    mwSize const *srcSize = mxGetDimensions( src );
    int N = (srcSize[0] == 1 ? srcSize[1] : srcSize[0]);

    std::cerr << "Src: " << srcSize[0] << "x" << srcSize[1] << std::endl;
    std::cerr << "N = " << N << std::endl;

    //-------------------------------------------------------------------------
    std::cerr << "Configuring output...\n";
    
    plhs[0] = mxCreateNumericArray(kNDims, srcSize, mxSINGLE_CLASS, mxREAL);
    float *p_dest = (float*)mxGetData( plhs[0] );

    std::cerr << std::endl;
    for (int i = 0; i < N; ++i)
	    std::cerr << "(" << i << ")\t@in = " << *(p_src+i) << "\t@out = " << *(p_dest+i) << std::endl;*/

    return;
}
