/*
###################################################
###### HR3D: Sparse lightfield times B_t
 */

//Includes
#include <iostream>
#include <string>
#include <cassert>
#include "matrix_factorization.h"
#include "mex.h"
#include "matrix.h"

#include <stdio.h>

//####################################
//#### Main mex entry point 
//####################################

void mexFunction(int nlhs, mxArray *plhs[], /* Output variables */
		 int nrhs, const mxArray *prhs[]) /* Input variables */
{
    //LBt = LmultBt(single(samples), int32(ch), single(B), int32(layerInt))
	
    //###################################################
    //####### Get inputs
    
    //#### All in *row-vec format* ####
    if( nlhs != 1 || nrhs != 4 )
	    mexErrMsgTxt("Wrong input/output parameters.");
    
    //Get pointers and data
    //Samples
    float * samples = (float*)mxGetData(prhs[0]);
    const int *samples_dims = mxGetDimensions(prhs[0]);
    int num_samples = samples_dims[1];
    
    //Channel
    int ch = *((int*)mxGetData(prhs[1]));
    if( ch < 1 || ch > samples_dims[0] )
	    mexErrMsgTxt("Channel out of range.\n");
    
    //B
    float * B = (float*)mxGetData(prhs[2]);
    const int  *B_dims = mxGetDimensions(prhs[2]);
    int B_sz_t = B_dims[0];
    int B_sz_layer = B_dims[1];
    
    //Layer int
    int * layerInt = (int*)mxGetData(prhs[3]);
    const int *layerInt_dims = mxGetDimensions(prhs[3]);
    
    //Sanity
    if( layerInt_dims[0] != 6 ||
        layerInt_dims[1] != num_samples )
	    mexErrMsgTxt("Inputs have wrong size.\n");
    
    //Debug	
    /*
    printf("num_samples %d, layerdim %d, Ch %d, Bszt %d Bszlayer %d \n", 
    		num_samples, layerInt_dims[0], ch, B_sz_t, B_sz_layer);
	    
    printf("samples\n\n");
    for(int i = 0; i < num_samples * samples_dims[0]; i++)
      printf("samples[i] = %3.3f\n", samples[i]);
    
    
    printf("B\n\n");
    for(int i = 0; i < B_sz_t * B_sz_layer; i++)
      printf("B[i] = %3.3f\n", B[i]);
    
    printf("layerInt\n\n");
    for(int i = 0; i < num_samples * 6; i++)
      printf("layerInt[i] = %d\n", layerInt[i]);
    */
    
    //###################################################
    //####### Allocate result
    
    int ndim = 2;
    int dims[2];
    dims[0] = B_sz_layer;
    dims[1] = B_sz_t;
    
    plhs[0] = mxCreateNumericArray(ndim, dims, mxSINGLE_CLASS, mxREAL);
    if (plhs[0] == NULL)
	mexErrMsgTxt("Could not create output mxArray.\n");
    
    //output
    float *res = (float *)mxGetPr(plhs[0]);
    
    //###################################################
    //####### Compute result
    
    LmultBt_cuda( B, samples, layerInt, res, 
		  ch,
		  num_samples, samples_dims[0],
		  B_sz_layer, B_sz_t);
    
    return;
}
