/*
###################################################
###### HR3D: Sparse matrix factorization
 */

//Includes
#include <iostream>
#include <string>
#include <cassert>
#include "matrix_factorization.h"
#include "mex.h"
#include "matrix.h"

//####################################
//#### Main mex entry point 
//####################################

void mexFunction(int nlhs, mxArray *plhs[], /* Output variables */
		 int nrhs, const mxArray *prhs[]) /* Input variables */
{
    //A, B = sparse_matrix_factorization(single(samples), int32(ch), single(A), single(B), int32(layerInt))
	
    //###################################################
    //####### Get inputs
    
    //#### All in *row-vec format* ####
    if( nlhs != 2 || nrhs != 5 )
	    mexErrMsgTxt("Wrong input/output parameters.");
    
    //Get pointers and data
    //Samples
    float * samples = (float*)mxGetData(prhs[0]);
    const int *samples_dims = mxGetDimensions(prhs[0]);
    int num_samples = samples_dims[1];
    
    //Channel
    int ch = *((int*)mxGetData(prhs[1]));
    if( ch < 1 || ch > samples_dims[0] )
	    mexErrMsgTxt("Channel out of range.\n");
	
	//A
    float * A = (float*)mxGetData(prhs[2]);
    const int  *A_dims = mxGetDimensions(prhs[2]);
    int A_sz_layer = A_dims[0];
    int A_sz_t = A_dims[1];
    
    //B
    float * B = (float*)mxGetData(prhs[3]);
    const int  *B_dims = mxGetDimensions(prhs[3]);
    int B_sz_t = B_dims[0];
    int B_sz_layer = B_dims[1];
    
    //Layer int
    int * layerInt = (int*)mxGetData(prhs[4]);
    const int *layerInt_dims = mxGetDimensions(prhs[4]);
    
    //Sanity
    if( layerInt_dims[0] != 6 ||
        layerInt_dims[1] != num_samples )
	    mexErrMsgTxt("Inputs have wrong size.\n");
    
    //###################################################
    //####### Allocate result
		
	int ndim = 2;
    int dims[2];
    dims[0] = A_sz_layer;
    dims[1] = A_sz_t;
    
    plhs[0] = mxCreateNumericArray(ndim, dims, mxSINGLE_CLASS, mxREAL);
    if (plhs[0] == NULL)
		mexErrMsgTxt("Could not create output mxArray.\n");
    
    //output
    float *resA = (float *)mxGetPr(plhs[0]);
    
    dims[0] = B_sz_t;
    dims[1] = B_sz_layer;
    
    plhs[1] = mxCreateNumericArray(ndim, dims, mxSINGLE_CLASS, mxREAL);
    if (plhs[1] == NULL)
		mexErrMsgTxt("Could not create output mxArray.\n");
    
    //output
    float *resB = (float *)mxGetPr(plhs[1]);
    
    //###################################################
    //####### Compute result
    
    matrix_factorization_cuda( A, B, samples, layerInt, resA, resB, 
		  ch,
		  num_samples, samples_dims[0],
		  A_sz_layer, A_sz_t,
		  B_sz_layer, B_sz_t);
    
    return;
}
