
/*
    Lightfield film contains lightfield angular and spatial resolutions.
    Currently reimplements image-film functionality with lightfield data description.
 */

#if defined(_MSC_VER)
#pragma once
#endif

#ifndef PBRT_FILM_LIGHTFIELD_FILM_H
#define PBRT_FILM_LIGHTFIELD_FILM_H

// film/lightfield.h*
#include "pbrt.h"
#include "film.h"
#include "sampler.h"
#include "filter.h"
#include "paramset.h"

#include <vector>


// ImageFilm Declarations
class LightfieldFilm : public Film {
public:
    // ImageFilm Public Methods
    LightfieldFilm(int xres, int yres,
		  float aMinX, float aMaxX, int rAX,
		  float aMinY, float aMaxY, int rAY,
		  float rayOffset,
		  int tRes,
		  const string &filename, bool allocData = true);
    
    ~LightfieldFilm();
    
    void AddSample(const CameraSample &s, const Spectrum &L);
    void AddSample(const LFSamplePixel &s);
    void Splat(const CameraSample &s, const Spectrum &L);
    void GetSampleExtent(int *xstart, int *xend,
                                 int *ystart, int *yend) const;
    void GetPixelExtent(int *xstart, int *xend,
                                int *ystart, int *yend) const;
    void WriteImage(float splatScale);
    void WriteImage(float splatScale, int ax, int ay);
    void WriteImageScanLine(float splatScale, int y, int a_y);
    void updateSize(int xres, int yres,
		  float aMinX, float aMaxX, int rAX,
		  float aMinY, float aMaxY, int rAY);
    
    //Data access
    void GetPixels(float* rgb, float splatScale);

    // LightfieldFilm Data
    float angleMinX; float angleMaxX; int resAngleX;	//All angles in degrees
    float angleMinY; float angleMaxY; int resAngleY;
    float rayOffset;
    int timeRes;
    string filename;
    
private:
    //Private functions
    int l2px(int x, int y, int ax, int ay);
  
    // LightfieldFilm Private Data
    struct Pixel {
        Pixel() {
            for (int i = 0; i < 3; ++i)
	      Lxyz[i] = splatXYZ[i] = 0.f;
	      weightSum = 0.f;
        }
        float Lxyz[3];
        float weightSum;
        float splatXYZ[3];
        float pad;
    };
    std::vector<Pixel> *pixels;
};

LightfieldFilm *CreateLightfieldFilm(const ParamSet &params, bool allocData);

#endif // PBRT_FILM_LIGHTFIELD_FILM_H
