
/*
    Lightfield reconstruction with a time-multiplexed layered attenuator.
 */

#if defined(_MSC_VER)
#pragma once
#endif

#ifndef PBRT_LIGHTFIELD_RECONSTUCTION_TIMEMULTIPLEXED_LAYERED_ATTENUATOR_H
#define PBRT_LIGHTFIELD_RECONSTUCTION_TIMEMULTIPLEXED_LAYERED_ATTENUATOR_H

// film/lightfield.h*
#include "pbrt.h"
#include "lightfield_reconstruction.h"
#include "sampler.h"
#include "filter.h"
#include "paramset.h"

#include <vector>


// LightfieldReconstructionTimeLayerAtt Declarations
class LightfieldReconstructionTimeLayerAtt: public LightfieldReconstruction {
public:
    // LightfieldReconstructionTimeLayerAtt Public Methods
    LightfieldReconstructionTimeLayerAtt(int xres, int yres,
				     float aMinX, float aMaxX, int rAX,
				     float aMinY, float aMaxY, int rAY,
				  
				    float OX, float OY, float OZ,
				    float lightfieldSzX, float lightfieldSzY,
				    
				    int layerResX, int layerResY, int layerResZ,
				    float layerOX, float layerOY, float layerOZ,
				    float layerSzX, float layerSzY, 
				    float layerDst,
				    
				    float scRec,
				      
				    int basisFunc,
					
				    int timeStps, 
				    std::vector<int> *lyrSubSet);
    
    ~LightfieldReconstructionTimeLayerAtt();
    
    //Size update lightfield reconstructon
    void updateSize(int xres, int yres,
		    int rAX, int rAY);
    
    void updateLayerParams(float OX, float OY, float OZ,	  
			  float lightfieldSzX, float lightfieldSzY,
			  int layerResX, int layerResY, int layerResZ,
			  float layerOX, float layerOY, float layerOZ,
			  float layerSzX, float layerSzY, 
			  float layerDst,
			  float scRec,
			  int basisFunc,
			  int timeStps, 
			  std::vector<int> *lyrSubSet);
    
    //Set float data 
    //Input index (y, x, time, color, layerNum) matlab index)
    void SetData(float* rgb_data);
    
    //Data access
    void GetPixel(const LFSamplePixel &s, float* rgb_val);
    
    //Output functions
    void WriteImage(int numLayer, int time, string filename); //Layers
    void GetView(float* view_rgb, int ax, int ay, int dx, int dy);
    
    
    private:
      
    //Private functions
    int i2px(int x, int y, int t, int layerNum);
    void computeDerivedParameters();
    
    //Lighfield param
    float angleMinX;
    float angleMaxX;
    float angleMinY;
    float angleMaxY;
    
    float origX; 
    float origY;
    float origZ;
    
    float lightfieldSizeX;
    float lightfieldSizeY;
    
    //Layer parameters
    int resLayerX;
    int resLayerY;
    int resLayerZ;
    
    float layerOrigX; 
    float layerOrigY;
    float layerOrigZ;
    
    float layerSizeX;
    float layerSizeY;
    
    float layerDist;
	
    int timeSteps;
    std::vector<int> layerSubSet;
    
    float scaleRec;

    int basisFunction;
    
    //Derived parameters
    float angleMinXV;
    float angleMaxXV;
    float angleMinYV;
    float angleMaxYV;
    
    float lightFieldPixelSizeX;
    float lightFieldPixelSizeY;
        
    float lookupScaleX;
    float lookupScaleY;
    
    // Layer data
    float* pixels;
};

#endif // PBRT_LIGHTFIELD_RECONSTUCTION_TIMEMULTIPLEXED_LAYERED_ATTENUATOR_H
