%Reset
clear all;
close all;

%Filename
fn = '../../scenes/killeroo-simple.pbrt';

%Define sampling params
numChains = 10;
chainLength = 500;
maxAcceptedSamplesPerChains = 500;
mutation_sigma_spatial = 1.0;
mutation_sigma_angular = 1.0;
thresh_residual = 0.01;

sampling_params = single([numChains; chainLength; maxAcceptedSamplesPerChains; ...
                          mutation_sigma_spatial; mutation_sigma_angular; thresh_residual]);

%Load 
[pbrt_ptr, l_sz] = mex_pbrt('create', fn, sampling_params);

%Convert
resX = l_sz(1);	
resY = l_sz(2);	
angleMinX = l_sz(3);
angleMaxX = l_sz(4);
resAngleX = l_sz(5);
angleMinY = l_sz(6);
angleMaxY = l_sz(7);
resAngleY = l_sz(8);

%Create lightfieldrec
lrec = zeros(resY, resX, resAngleX * resAngleY * 3);

%Load from file
%{
rec = load('rec.mat');
rec = rec.rec;
imshow(rec);
lrec = reshape(shiftdim(rec,1), resY, resX, resAngleX * resAngleY * 3);
%}

%Convert to single !
lrec = single(lrec);

%Do a number of sampling iterations
rec = zeros(resAngleX, resX, 3);
for iter = 0:5

    %Do something
    render_params = single([chainLength; mutation_sigma_spatial; ...
                            mutation_sigma_angular; thresh_residual]);
    [meas, indx] = mex_pbrt('sample', pbrt_ptr,lrec, render_params);

    %1D Generate result and add samples
    overwrited = false;
    for s = 1:size(indx,2)
       for ch = 0:2
           
           %Get idx
           x = double(indx(1,s));
           y = double(indx(2,s));
           ax = double(indx(3,s));
           ay = double(indx(4,s));
           %ch_angle_idx = ay * resAngleX * 3 + ax * 3 + ch; 
           
           %Write
           if( rec(ax + 1, x + 1, ch + 1) ~= 0 && ~overwrited)
               warning('Overwrited !!!')
               overwrited = true;
           end

           rec(ax + 1, x + 1, ch + 1) = double(meas(ch + 1,s));
           
       end
    end

    %show
    imshow( rec ); 
    title(sprintf('Iter %d', iter));

    %wait
    w = waitforbuttonpress; 

end


%Change resolution
l_newsize = single([resX*2; resY; angleMinX; angleMaxX; resAngleX*1; angleMinY; angleMaxY; resAngleY]);
[l_sz] = mex_pbrt('changeres', pbrt_ptr, l_newsize);

%Convert
resX = l_sz(1);	
resY = l_sz(2);	
angleMinX = l_sz(3);
angleMaxX = l_sz(4);
resAngleX = l_sz(5);
angleMinY = l_sz(6);
angleMaxY = l_sz(7);
resAngleY = l_sz(8);

%Create changed lightfieldrec
lrec = zeros(resY, resX, resAngleX * resAngleY * 3);

%Convert to single !
lrec = single(lrec);

%Do a number of sampling iterations
rec = zeros(resAngleX, resX, 3);
for iter = 0:20

    %Do something
    render_params = single([chainLength; mutation_sigma_spatial; ...
                            mutation_sigma_angular; thresh_residual]);
    [meas, indx] = mex_pbrt('sample', pbrt_ptr,lrec, render_params);

    %1D Generate result and add samples
    overwrited = false;
    for s = 1:size(indx,2)
       for ch = 0:2
           
           %Get idx
           x = double(indx(1,s));
           y = double(indx(2,s));
           ax = double(indx(3,s));
           ay = double(indx(4,s));
           %ch_angle_idx = ay * resAngleX * 3 + ax * 3 + ch; 
           
           %Write
           if( rec(ax + 1, x + 1, ch + 1) ~= 0 && ~overwrited)
               warning('Overwrited !!!')
               overwrited = true;
           end

           rec(ax + 1, x + 1, ch + 1) = double(meas(ch + 1,s));
           
       end
    end

    %show
    imshow( rec ); 
    title(sprintf('Iter %d', iter));

    %wait
    w = waitforbuttonpress; 

end

%Destroy pbrt
mex_pbrt('destroy', pbrt_ptr);

%{
%Generate result and add samples
rec = zeros(resY, resX, resAngleX * resAngleY * 3);
for s = 1:size(indx,2)
   for ch = 0:2
       %Get idx
       x = double(indx(1,s));
       y = double(indx(2,s));
       ax = double(indx(3,s));
       ay = double(indx(4,s));
       ch_angle_idx = ay * resAngleX * 3 + ax * 3 + ch; 
       
       rec(y + 1,x + 1,ch_angle_idx + 1) = double(meas(ch + 1,s));
   end
end

%show
for i = 1:3:size(rec,3)
    imshow( rec(:,:,i+(0:2) )); 
    w = waitforbuttonpress;  
end
%}
