/*
    Adaptive Lightfield sampler renderer
 */

#if defined(_MSC_VER)
#pragma once
#endif

#ifndef PBRT_RENDERERS_ADAPTIVELFSAMPLERRENDERER_H
#define PBRT_RENDERERS_ADAPTIVELFSAMPLERRENDERER_H

// renderers/adaptiveLFsamplerrenderer.h*
#include "pbrt.h"
#include "renderer.h"
#include "parallel.h"
#include "sampler.h"
#include "montecarlo.h"
#include <film/lightfield_film.h>
#include "lightfield_reconstruction.h"

//#include <nanoflann.hpp>

#include <stdio.h>
#include <Judy.h>

class SampleSet
{
public:

	std::vector<LFSamplePixel> *pts;
	
	//Terminal condition
	std::vector<bool> *addedNewSample;
	int newSampleBegin;
	bool early_exit;
	
	SampleSet(int initsize = 0)
	{
	    pts = new std::vector<LFSamplePixel>( initsize );
	    
	    //Terminals
	    addedNewSample = NULL;
	    newSampleBegin = 0;
	    early_exit = false;
	}
	
	~SampleSet()
	{
	    delete pts;
	    pts = NULL;
	}

	/*
	// Must return the number of data points
	inline size_t kdtree_get_point_count() const { return (*pts).size(); }

	// Returns the distance between the vector "p1[0:size-1]" and the data point with index "idx_p2" stored in the class:
	inline int kdtree_distance(const int *p1, const size_t idx_p2,size_t size) const
	{
	    const int d0=p1[0] - (*pts)[idx_p2].x;
	    const int d1=p1[1] - (*pts)[idx_p2].y;
	    const int d2=p1[2] - (*pts)[idx_p2].ax;
	    const int d3=p1[3] - (*pts)[idx_p2].ay;
	    
	    return d0*d0 + d1*d1 + d2*d2 + d3*d3;
	}

	// Returns the dim'th component of the idx'th point in the class:
	// Since this is inlined and the "dim" argument is typically an immediate value, the
	//  "if/else's" are actually solved at compile time.
	inline int kdtree_get_pt(const size_t idx, int dim) const
	{
	    if (dim==0) return (*pts)[idx].x;
	    else if (dim==1) return (*pts)[idx].y;
	    else if (dim==2) return (*pts)[idx].ax;
	    else return (*pts)[idx].ay;
	}
	
	inline 	void setup_terminal_cond(  int newSampleBegin, std::vector<bool> *addedNewSample  )
	{
	    this->newSampleBegin = newSampleBegin;
	    this->addedNewSample = addedNewSample;
	}
	
	//Structures for a point to be solved
	inline 	bool point_found(  const size_t idx ) const
	{
	    //Check for early exit
	    if( idx < newSampleBegin ||
		addedNewSample->at( idx - newSampleBegin ) )
		return true;
	    else
		return false;
	}

	// Optional bounding-box computation: return false to default to a standard bbox computation loop.
	template <class BBOX>
	bool kdtree_get_bbox(BBOX &bb) const { return false; }
	*/

};

/*
// construct a kd-tree index:
typedef nanoflann::KDTreeSingleIndexAdaptor<
	nanoflann::L2_Simple_Adaptor<int, SampleSet> ,
	SampleSet,
	4 
	> LF_kd_tree;
*/

// #### Adaptive Sampling declarations

// Sampling chain
class LFSamplingChain {
public:
    // #### LFSamplingChain Public Methods
    LFSamplingChain(int maxRecSize = 0);
    LFSamplingChain(const LFSamplingChain &copyin);
    ~LFSamplingChain();
    
    bool record(LFSamplePixel& acceptedSample);
    void reset();
    
    
    // #### LFSamplingChain data
    LFSamplePixel sample; 			// Current sample
    
    std::vector<LFSamplePixel> *recorded; 	// Last recorded samples
    int maxRecordedSize;			// Maximum number of allowed records
    
    int idx;					// Current index in chain
    bool initialized;
};

// #### Adaptive Sampling rendering

// AdaptiveLFSamplerRenderer Declarations
class AdaptiveLFSamplerRenderer : public Renderer {
public:
    // AdaptiveLFSamplerRenderer Public Methods
    AdaptiveLFSamplerRenderer(Sampler *s, Camera *c, SurfaceIntegrator *si,
                    VolumeIntegrator *vi, bool visIds,
		    LightfieldReconstruction * lfRec,
		    int duplicateChk,
		    int numChains, int chLength, int maxAcceptedSamplesPerChains,
		    float m_sigma_spatial, float m_sigma_angular, float t_residual, 
		    int nChDuplicateChecks, bool rplSamples,
		    bool smplTime = false,
		    bool smplFull = false);
    
    ~AdaptiveLFSamplerRenderer();
    
    void Render(const Scene *scene);
    void Render(const Scene *scene, float time );
    
    void resetChains();
    
    void removeDuplicates();
    
    void updateSamplingParams(int chLength, 
			      float m_sigma_spatial, 
			      float m_sigma_angular, 
			      float t_residual,
			      int nChDuplicateChecks,
			      bool rplSamples);
    
    void updateSize();
    
    Spectrum Li(const Scene *scene, const RayDifferential &ray,
        const Sample *sample, RNG &rng, MemoryArena &arena,
        Intersection *isect = NULL, Spectrum *T = NULL) const;
    Spectrum Transmittance(const Scene *scene, const RayDifferential &ray,
        const Sample *sample, RNG &rng, MemoryArena &arena) const;
	
    //1D test case
    void set1DtestData(float *data );
    float* lightfield1DtestData;
    
    //Intitialized intitial structures
    bool initializedRenderStructures;
	
    //Adaptive sampling data
    std::vector<LFSamplingChain> *chains;
    LightfieldReconstruction * lightfieldRec;
    
    int chainLength;
    float mutation_sigma_spatial;
    float mutation_sigma_angular; 
    float thresh_residual;
    int numChainDuplicateChecks;
    
    //Sample time
    bool sampleTime;
    
    //Sample everywhere
    bool sampleFull;
    
    //Holder for all samples
    SampleSet* sampleSet;
    int newSampleBegin;
    
    int totalRecordedSamples;
    int totalTestedSamples;
    
    //Record checking structures
    int duplicateCheck;
    bool replaceSamples;
    
    int* recordedSamples;
    Pvoid_t recordedSamplesJArray;
    //LF_kd_tree* recordedKDtree;
    
    //Timings
    std::vector<float> samplingTimings;
    
    
private:
    // AdaptiveLFSamplerRenderer Private Data
    bool visualizeObjectIds;
    Sampler *sampler;
    Camera *camera;
    SurfaceIntegrator *surfaceIntegrator;
    VolumeIntegrator *volumeIntegrator;
    
};

// Sampling data Declarations
class SamplingData {
public:
    // All structures needed for single LF pixel measurements
    Sampler *sampler;
    MemoryArena* arena;
    RNG* rng;
    
    Sample *samples;
    RayDifferential *rays;
    Spectrum *Ls;
    Spectrum *Ts;
    Intersection *isects;
    
    LightfieldFilm *film;
};

// SamplerRendererTask Declarations
class AdaptiveLFSamplerRendererTask : public Task {
public:
    // SamplerRendererTask Public Methods
    AdaptiveLFSamplerRendererTask(const Scene *sc, Renderer *ren, Camera *c,
                        ProgressReporter &pr, Sampler *ms, Sample *sam, 
                        bool visIds, 
			LightfieldReconstruction * lfRec,
			std::vector<LFSamplingChain> *schains, int chLength, 
			float m_sigma_spatial, float m_sigma_angular, float t_residual,
			int duplicateChk, int duplicateChainChk,
			int *rsamples, float* lf1DtestData, 
			bool smplTime, bool smplFull, SampleSet* smpleSet,
			int tn, int tc);
    void Run();
    
    //Recorded samples since last run (public)
    int totalRecordedSamples;
    int totalTestedSamples;
    
private:
    
    //#### Adaptive sampling functionality
    void EvaluateLightfield( LFSamplePixel* sample, SamplingData* sD );
    
    //Sampling
    float sampleChain(LFSamplingChain *chain, 
		      int   n_samples_chain,
		      float mutation_sigma_spatial,
		      float mutation_sigma_angular,
		      float thresh_residual,
		      SamplingData* sD);

    float getFunctionVal( LFSamplePixel* sample );
    
    bool accepted( LFSamplePixel* sample, LFSamplingChain *chain, SamplingData* sD );
 
    bool mutate( LFSamplePixel* in, LFSamplePixel* out, LFSamplingChain *chain, SamplingData* sD,
		float mutation_sigma_spatial,
		float mutation_sigma_angular );
    
    bool record( LFSamplingChain *chain, 
		 LFSamplePixel* sample, 
		 SamplingData* sD );
    
    
    //#### Local helper functions
    void ComputeSubChains(int num, int count, 
			  int *newCStart, int *newCEnd) const;
			  
    void ComputeSubWindow(int num, int count, int *newXStart,
        int *newXEnd, int *newYStart, int *newYEnd) const;
			  
    // SamplerRendererTask Private Data
    const Scene *scene;
    const Renderer *renderer;
    Camera *camera;
    Sampler *mainSampler;
    ProgressReporter &reporter;
    Sample *origSample;
    bool visualizeObjectIds;
    int taskNum, taskCount;
    
    bool sampleTime;
    bool sampleFull; 
    SampleSet* sampleSet;
    
    std::vector<LFSamplingChain> *chains;
    LightfieldReconstruction * lightfieldRec;
    
    int chainLength;
    float mutation_sigma_spatial;
    float mutation_sigma_angular; 
    float thresh_residual;
    int duplicateChainCheck;
    
    int duplicateCheck;
    int* recordedSamples;
    
    //Debug tests
    int dStartX;
    int dEndX;
    int dStartY;
    int dEndY;
    
    //1D test case
    float* lightfield1DtestData;
};

#endif // PBRT_RENDERERS_ADAPTIVELFSAMPLERRENDERER_H
