
/*
    Low discrepancy filtering for lightfields.
 */

#if defined(_MSC_VER)
#pragma once
#endif

#ifndef PBRT_SAMPLERS_LOWDISCREPANCYLIGHTFIELD_H
#define PBRT_SAMPLERS_LOWDISCREPANCYLIGHTFIELD_H

// samplers/lowdiscrepancy.h*
#include "sampler.h"
#include "paramset.h"
#include "film/lightfield_film.h"
#include "parallel.h"

// LDSampler Declarations
class LDSamplerLightfield : public Sampler {
public:
    // LDSampler Public Methods
    LDSamplerLightfield(int xstart, int xend, int ystart, int yend, int xAend, int yAend,
              int nsamp, float sopen, float sclose);
    ~LDSamplerLightfield();
    Sampler *GetSubSampler(int num, int count);
    int RoundSize(int size) const { return RoundUpPow2(size); }
    int GetMoreSamples(Sample *sample, RNG &rng);
    int GetLFPixelSamples( int x, int y, int ax, int ay,
			 Sample *samples, RNG &rng);
    void updateSize( int xres, int yres,
		    int rAX, int rAY);
    int MaximumSampleCount() { return nPixelSamples; }
private:
    // LDSampler Private Data
    int xPos, yPos, xAPos, yAPos;
    
    int nPixelSamples;
    int xAnglePixelEnd, yAnglePixelEnd;
    
    
    float *sampleBuf;
};


LDSamplerLightfield *CreateLowDiscrepancySamplerLightfield(const ParamSet &params, const LightfieldFilm *film,
        const Lightfield *lf);

#endif // PBRT_SAMPLERS_LOWDISCREPANCYLIGHTFIELD_H
